#include "gideros.h"

@interface MyTimer : NSObject
{
	NSTimer* timer;
	lua_State* L;
}

@end

@implementation MyTimer

- (id)initWithLuaState:(lua_State*)theL
{
    self = [super init];
    if (self)
	{
		L = theL;
		timer = [NSTimer scheduledTimerWithTimeInterval:1.0
												 target:self
											   selector:@selector(onTick:)
											   userInfo:nil
												repeats:YES];	
	}
    return self;
}

-(void)onTick:(NSTimer *)timer
{
	lua_getglobal(L, "onesec");
	if (!lua_isnil(L, -1))
	{
		lua_getfield(L, -1, "dispatchEvent");	// get the dispatchEvent function of onesec
		lua_pushvalue(L, -2);	// duplicate onesec, this will be the 1st parameter of dispatchEvent
		
		// create an Event object (this will be the 2nd parameter of dispatchEvent)
		lua_getglobal(L, "Event");		// get the global Event table
		lua_getfield(L, -1, "new");		// get its new function
		lua_remove(L, -2);				// remove the global Event table
		lua_pushstring(L, "complete");
		lua_call(L, 1, 1);				// call as Event.new("complete")
		
		lua_call(L, 2, 0);	// call dispatchEvent with onesec and event
	}
	lua_pop(L, 1);	// pop nil or onesec
}

- (void)invalidate
{
	[timer invalidate];
}

@end


static int destruct(lua_State* L)
{
	void* ptr = *(void**)lua_touserdata(L, 1);
	GReferenced* object = static_cast<GReferenced*>(ptr);
	GEventDispatcherProxy* proxy = static_cast<GEventDispatcherProxy*>(object->proxy());
	
	proxy->unref();
	
	return 0;
}

static int loader(lua_State* L)
{
	const luaL_Reg functionlist[] = {
		{NULL, NULL},
	};
	
	g_createClass(L, "OneSec", "EventDispatcher", NULL, destruct, functionlist);

	GEventDispatcherProxy* proxy = new GEventDispatcherProxy;
	g_pushInstance(L, "OneSec", proxy->object());

	lua_pushvalue(L, -1);
	lua_setglobal(L, "onesec");
	
	return 1;
}

static MyTimer* timer = nil;

static void g_initializePlugin(lua_State *L)
{
	lua_getglobal(L, "package");
	lua_getfield(L, -1, "preload");
	
	lua_pushcfunction(L, loader);
	lua_setfield(L, -2, "onesec");
	
	lua_pop(L, 2);
	
	timer = [[MyTimer alloc] initWithLuaState:L];
}

static void g_deinitializePlugin(lua_State *L)
{
	[timer invalidate];
	[timer release];
	timer = nil;
}

REGISTER_PLUGIN("OneSec", "1.0")
